@@ -76,13 +76,14 @@ typedef struct
   int dont_split;
   int orphan_control;
   int first_line_indent;	/* At true margin if negative */
-  sem_act translate;
+  char *translation_table;
   int skip_number_lines;	/*Don't write on lines with page numbers */
   StyleFormat format;
   BrlPageNumFormat brlNumFormat;
   int newpage_before;
   int newpage_after;
   int righthand_page;
+  int newline_after;
 } StyleType;
 
 #ifdef _WIN32
@@ -94,9 +95,6 @@ typedef struct
 #define CHARSIZE sizeof (widechar)
 #endif
 
-#define CELLWIDTH 5
-#define NORMALLINE 8
-#define WIDELINE 10
 #define BUFSIZE 8192
 #define MAX_LENGTH BUFSIZE - 4
 #define MAX_TRANS_LENGTH 2 * BUFSIZE - 4
@@ -124,7 +122,12 @@ typedef enum
   textDevice = 0,
   browser,
   utd,
-  pef
+  pef,
+  transinxml,
+  volumes,
+  brf,
+  volumesPef,
+  volumesBrf
 } FormatFor;
 
 typedef enum
@@ -167,19 +170,30 @@ typedef struct
   TextFormat back_text;
   int back_line_length;
   FormatFor format_for;
+  FormatFor orig_format_for;
   int contents;
   int has_contentsheader;
   unsigned int mode;
+  unsigned int orig_mode;
+  unsigned int config_mode;
   translationModes louis_mode;
   int emphasis;
+  double dpi;
+  int cell_width;
+  int normal_line;
+  int wide_line;
   int debug;
   int paper_width;
   int paper_height;
+  int page_left;
+  int page_top;
+  int page_right;
+  int page_bottom;
+  int vert_line_pos;
   int top_margin;
   int bottom_margin;
   int left_margin;
   int right_margin;
-  int contains_utd;
   int cells_per_line;
   int lines_per_page;
   int beginning_braille_page_number;
@@ -242,6 +256,7 @@ typedef struct
   char line_fill;
   char lit_hyphen[5];
   char comp_hyphen[5];
+  char letsign[5];
   widechar running_head[MAXNAMELEN / 2];
   widechar footer[MAXNAMELEN / 2];
   int running_head_length;
@@ -254,6 +269,9 @@ typedef struct
   const char *edit_table_name;
   const char *semantic_files;
   const char *volume_sem;
+  const char *pef_sem;
+  const char *brf_sem;
+  const char *transinxml_sem;
   widechar print_page_number[MAXNUMLEN];
   widechar braille_page_string[MAXNUMLEN];
   char lineEnd[8];
@@ -292,7 +310,6 @@ int transcribe_paragraph (xmlNode * node, int action);
 int transcribe_chemistry (xmlNode * node, int action);
 int transcribe_graphic (xmlNode * node, int action);
 int transcribe_music (xmlNode * node, int action);
-int interpret_utd (xmlNode * node, NodeAction action);
 int compile_semantic_table (xmlNode * rootElement);
 sem_act set_sem_attr (xmlNode * node);
 sem_act get_sem_attr (xmlNode * node);
@@ -338,7 +355,8 @@ int end_style ();
 int find_action (const char **actions, const char *action);
 int find_group_length (const char groupSym[2], const char *groupStart);
 char *alloc_string (const char *inString);
-int write_outbuf ();
+char *alloc_string_if_not (const char *inString);
+int write_buffer (int from, int skip);
 int link_brl_node (xmlNode * node);
 void clean_semantic_table ();
 int back_translate_braille_string ();
@@ -347,4 +365,13 @@ int utf8_string_to_wc (const unsigned char *instr, int *inSize, widechar
 int *outSize);
 int wc_string_to_utf8 (const widechar * instr, int *inSize, unsigned
 		       char *outstr, int *outSize);
+void output_xml (xmlDoc *doc);
+int convert_utd ();
+int utd2brf (xmlNode * node);
+int utd2pef (xmlNode * node);
+int utd2transinxml (xmlNode * node);
+int utd2volumes (xmlNode * node);
+int hasAttrValue  (xmlNode *node, char *attrName, char *value);
+void kill_safely ();
+
 #endif /*louisutdml_h */
