@@ -56,6 +56,7 @@ typedef struct
 }
 FileInfo;
 
+static char lastConfigFileList[MAXNAMELEN];
 static double paperWidth;
 static double paperHeight;
 static double leftMargin;
@@ -86,8 +87,11 @@ configureError (FileInfo * nested, char *format, ...)
 char *
 alloc_string (const char *inString)
 {
-  int length = strlen (inString);
+  int length;
   char *newString;
+  if (inString == NULL)
+  return NULL;
+  length  = strlen (inString);
   if ((length + ud->string_buf_len) >= sizeof (ud->string_buffer))
     return NULL;
   newString = &ud->string_buffer[ud->string_buf_len];
@@ -96,6 +100,18 @@ alloc_string (const char *inString)
   return newString;
 }
 
+char *
+alloc_string_if_not (const char *inString)
+{
+  int alreadyStored;
+  if (inString == NULL)
+  return NULL;
+  alreadyStored = inString - ud->string_buffer;
+  if (alreadyStored >= 0 && alreadyStored < ud->string_buf_len)
+    return (char *) inString;
+  return (char *) alloc_string (inString);
+}
+
 int
 file_exists (const char *completePath)
 {
@@ -185,10 +201,11 @@ findTable (FileInfo * nested)
   char trialPath[MAXNAMELEN];
   if (!find_file (nested->value, trialPath))
     {
-      configureError (nested, "Table %s cannot be found.", nested->value);
+      configureError (nested, "Table '%s' cannot be found.", 
+      nested->value);
       return NULL;
     }
-  return alloc_string (trialPath);
+  return alloc_string_if_not (trialPath);
 }
 
 static int
@@ -220,7 +237,8 @@ controlCharValue (FileInfo * nested)
 	      decoded[decodedLength++] = 13;
 	      break;
 	    default:
-	      configureError (nested, "invalid value %s", nested->value);
+	      configureError (nested, "invalid value '%s'", 
+	      nested->value);
 	      return 0;
 	    }
 	  k++;
@@ -251,7 +269,8 @@ config_compileSettings (const char *fileName)
     return compileConfig (&nested);
   if (!find_file (fileName, completePath))
     {
-      configureError (NULL, "Can't find configuration file %s", fileName);
+      configureError (NULL, "Can't find configuration file '%s'", 
+      fileName);
       return 0;
     }
   if ((nested.in = fopen ((char *) completePath, "rb")))
@@ -261,7 +280,8 @@ config_compileSettings (const char *fileName)
     }
   else
     {
-      configureError (NULL, "Can't open configuration file %s", fileName);
+      configureError (NULL, "Can't open configuration file '%s'", 
+      fileName);
       return 0;
     }
   return 1;
@@ -312,6 +332,9 @@ parseLine (FileInfo * nested)
 {
   char *curchar = NULL;
   int ch = 0;
+  nested->action = NULL;
+  nested->value = NULL;
+  nested->value2 = NULL;
   while (getLine (nested))
     {
       nested->lineNumber++;
@@ -422,38 +445,78 @@ checkValues (FileInfo * nested, const char **values)
       break;
   if (values[k] == NULL)
     {
-      configureError (nested, "word %s in column 2 not recognized",
+      configureError (nested, "word '%s' in column 2 not recognized",
 		      nested->value);
       return NOTFOUND;
     }
   return atoi (values[k + 1]);
 }
 
+static unsigned int
+hexValue (FileInfo * nested, const char *digits)
+{
+  int length = strlen (digits);
+  int k;
+  unsigned int binaryValue = 0;
+  for (k = 0; k < length; k++)
+    {
+      unsigned int hexDigit = 0;
+      if (digits[k] >= '0' && digits[k] <= '9')
+	hexDigit = digits[k] - '0';
+      else if (digits[k] >= 'a' && digits[k] <= 'f')
+	hexDigit = digits[k] - 'a' + 10;
+      else if (digits[k] >= 'A' && digits[k] <= 'F')
+	hexDigit = digits[k] - 'A' + 10;
+      else
+	{
+	  configureError (nested, "invalid %d-digit hexadecimal number",
+			  length);
+	  return 0xffffffff;
+	}
+      binaryValue |= hexDigit << (4 * (length - 1 - k));
+    }
+  return binaryValue;
+}
+
+static unsigned int
+convertValue (FileInfo *nested, const char *number)
+{
+  if (number[0] == '0' && number[1] == 'x')
+    return hexValue (nested, &number[2]);
+  else if (number[0] == '1' && number[1] == '<')
+    {
+      int shift = atoi (&number[3]);
+      return 1 << shift;
+    }
+  else
+    return atoi (number);
+}
+
 static int
 orValues (FileInfo * nested, const char **values)
 {
   int result = 0;
   int k;
   int word = 0;
-  int wordLength = 0;
+  int wordLength;
   while (word < nested->valueLength)
     {
-      for (; word < nested->valueLength && nested->value[word] <= ' ';
-	   word++);
-      for (wordLength = 0; (word + wordLength) < nested->valueLength &&
-	   nested->value[word + wordLength] > ' '; wordLength++);
+      for (wordLength = 0; (word + wordLength) < nested->valueLength && 
+      nested->value[word + wordLength] 
+      > ' '
+	   && nested->value[word + wordLength] != ','; wordLength++);
       for (k = 0; values[k]; k += 2)
 	if (wordLength == strlen (values[k]) &&
 	    ignoreCaseComp (values[k], &nested->value[word], wordLength) == 0)
 	  {
-	    result |= atoi (values[k + 1]);
+	    result |= convertValue (nested, values[k + 1]);
 	    break;
 	  }
-      word += wordLength;
+      word += wordLength + 1;
     }
   if (result == 0)
     {
-      configureError (nested, "word %s in column 2 not recognized",
+      configureError (nested, "word '%s' in column 2 not recognized",
 		      nested->value);
       return NOTFOUND;
     }
@@ -476,7 +539,8 @@ checkSubActions (FileInfo * nested, const char **mainActions, const char
     {
       mainActionNumber = checkActions (nested, mainActions);
       if (mainActionNumber == NOTFOUND)
-	configureError (nested, "word %s in first column not recognized",
+	configureError (nested, 
+	"word '%s' in first column not recognized",
 			nested->action);
       return NOTFOUND;
     }
@@ -567,8 +631,7 @@ compileConfig (FileInfo * nested)
     "35",
     "leftMargin",
     "36",
-    "containsUtd",
-    "37",
+// available    "37",
     "volumeSem",
     "38",
     "braillePageNumber",
@@ -591,16 +654,28 @@ compileConfig (FileInfo * nested)
     "47",
     "emphasis",
     "48",
-    "mergeUnnumberedPages",
+    "paperWidth",
     "49",
-    "pageNumberTopSeparateLine",
+    "paperHeight",
     "50",
-    "pageNumberBottomSeparateLine",
+    "numberBraillePages",
     "51",
     "printPageNumbersInContents",
     "52",
     "braillePageNumbersInContents",
     "53",
+    "rightMargin",
+    "54",
+    "bottomMargin",
+    "55",
+    "mode",
+    "56",
+    "pefSem",
+    "57",
+    "transinxmlSem",
+    "58",
+    "brfSem",
+    "59",
     "style",
     "90",
     NULL
@@ -623,6 +698,11 @@ compileConfig (FileInfo * nested)
     "browser", "1",
     "utd", "2",
     "pef", "3",
+    "transInXml", "4",
+    "volumes", "5",
+    "brf", "6",
+    "volumesPef", "7"
+    "volumesBrf", "8",
     NULL
   };
 
@@ -632,6 +712,26 @@ compileConfig (FileInfo * nested)
     "bold", "4",
     "computer_braille", "8",
     "all", "15",
+    "none", "0x100",
+    NULL
+  };
+  static const char *configModes[] = {
+    /*liblouis modes */
+    "noContractions", "1",
+    "compbrlAtCursor", "2",
+    "dotsIO", "4",
+    "comp8Dots", "8",
+    "pass1Only", "16",
+    "compbrlLeftCursor", "32",
+    "otherTrans", "64",
+    "ucBrl", "128",
+    /*liblouisutdml modes */
+    "doInit", "1<<30",
+    "htmlDoc", "1<<29",
+    "notUC", "1<<28",
+    "notSync", "1<<27",
+    "utdInput", "1<<26",
+    "convertOnly", "1<<25",
     NULL
   };
 
@@ -642,9 +742,9 @@ compileConfig (FileInfo * nested)
       if (mainActionNumber == NOTFOUND)
 	{
 	  configureError (nested,
-			  "word %s in first column not recognized",
+			  "word '%s' in first column not recognized",
 			  nested->action);
-	  return 0;
+	  continue;
 	}
     choseMainAction:
       switch (mainActionNumber)
@@ -738,7 +838,7 @@ compileConfig (FileInfo * nested)
 	  ud->mathexpr_table_name = findTable (nested);
 	  break;
 	case 24:
-	  ud->top_margin = atoi (nested->value);
+	  topMargin = atof (nested->value);
 	  break;
 	case 25:
 	  if (entities)
@@ -751,6 +851,8 @@ compileConfig (FileInfo * nested)
 	  strcpy (ud->xml_header, nested->value);
 	  break;
 	case 26:
+	  if (nested->value == NULL)
+	    break;
 	  if (!entities)
 	    strcat (ud->xml_header, "<!DOCTYPE entities [\n");
 	  entities = 1;
@@ -810,15 +912,11 @@ compileConfig (FileInfo * nested)
 	    ud->debug = k;
 	  break;
 	case 36:
-	  ud->left_margin = atoi (nested->value);
-	  break;
-	case 37:
-	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
-	    ud->contains_utd = k;
+	  leftMargin = atof (nested->value);
 	  break;
+// available	case 37:
 	case 38:
 	  ud->volume_sem = alloc_string (nested->value);
-	  ud->format_for = utd;
 	  break;
 	case 39:
 	  ud->braille_page_number = atoi (nested->value);
@@ -860,21 +958,15 @@ compileConfig (FileInfo * nested)
 	    ud->emphasis = k;
 	  break;
 	case 49:
-	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
-	    {
-	      ud->merge_unnumbered_pages = k;
-	    }
+	  paperWidth = atof (nested->value);
 	  break;
 	case 50:
-	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
-	    {
-	      ud->page_number_top_separate_line = k;
-	    }
+	  paperHeight = atof (nested->value);
 	  break;
 	case 51:
 	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
 	    {
-	      ud->page_number_bottom_separate_line = k;
+	      ud->number_braille_pages = k;
 	    }
 	  break;
 	case 52:
@@ -885,6 +977,25 @@ compileConfig (FileInfo * nested)
 	  if ((k = checkValues (nested, yesNo)) != NOTFOUND)
 	    ud->braille_page_numbers_in_contents = k;
 	  break;
+	case 54:
+	  rightMargin = atof (nested->value);
+	  break;
+	case 55:
+	  bottomMargin = atof (nested->value);
+	  break;
+	case 56:
+	  if ((k = orValues (nested, configModes)) != NOTFOUND)
+	    ud->config_mode = k;
+	  break;
+	case 57:
+	  ud->pef_sem = alloc_string (nested->value);
+	  break;
+	case 58:
+	  ud->transinxml_sem = alloc_string (nested->value);
+	  break;
+	case 59:
+	  ud->brf_sem = alloc_string (nested->value);
+	  break;
 	case 90:
 	  {
 	    static const char *actions[] = {
@@ -896,7 +1007,7 @@ compileConfig (FileInfo * nested)
 	      "2",
 	      "firstLineIndent",
 	      "3",
-	      "translate",
+	      "translattionTable",
 	      "6",
 	      "skipNumberLines",
 	      "7",
@@ -918,6 +1029,8 @@ compileConfig (FileInfo * nested)
 	      "15",
 	      "orphanControl",
 	      "16",
+	      "newlineAfter",
+	      "17",
 	      NULL
 	    };
 	    static const char *formats[] = {
@@ -934,10 +1047,23 @@ compileConfig (FileInfo * nested)
 	      "listColumns",
 	      "5",
 	      "listLines",
-	      "6", "computerCoded", "7", "contents", "8", NULL
+	      "6", 
+	      "computerCoded", 
+	      "7", 
+	      "contents", 
+	      "8", 
+	      NULL
 	    };
 	    static const char *pageNumFormats[] = {
-	      "normal", "0", "blank", "1", "p", "2", "roman", "3", NULL
+	      "normal", 
+	      "0", 
+	      "blank", 
+	      "1", 
+	      "p", 
+	      "2", 
+	      "roman", 
+	      "3", 
+	      NULL
 	    };
 	    StyleType *style;
 	    sem_act styleAction;
@@ -950,6 +1076,7 @@ compileConfig (FileInfo * nested)
 	    styleAction = find_semantic_number (nested->value);
 	    style = new_style (nested->value);
 	    style->action = styleAction;
+	    style->newline_after = 1;
 	    while (parseLine (nested))
 	      {
 		checkSubActions (nested, mainActions, actions);
@@ -972,17 +1099,7 @@ compileConfig (FileInfo * nested)
 		    style->first_line_indent = atoi (nested->value);
 		    break;
 		  case 6:
-		    switch ((k = find_semantic_number (nested->value)))
-		      {
-		      case contracted:
-		      case uncontracted:
-		      case compbrl:
-			style->translate = k;
-			break;
-		      default:
-			configureError (nested, "no such translation");
-			break;
-		      }
+		    style->translation_table = findTable (nested);
 		    break;
 		  case 7:
 		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
@@ -1023,6 +1140,10 @@ compileConfig (FileInfo * nested)
 		  case 16:
 		    style->orphan_control = atoi (nested->value);
 		    break;
+		  case 17:
+		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
+		      style->newline_after = k;
+		  break;
 		  default:
 		    configureError (nested, "Program error in readconfig.c");
 		    continue;
@@ -1062,7 +1183,7 @@ initConfigFiles (const char *firstConfigFile, char *fileName, const
   if (!config_compileSettings ("liblouisutdml.ini"))
     {
       if (!config_compileSettings ("canonical.cfg"))
-        return 0;
+	return 0;
     }
   return 1;
 }
@@ -1080,7 +1201,9 @@ read_configuration_file (const char *configFileList, const char
   int currentListPos = 0;
   errorCount = 0;
   /*Process logFileName later, after writeablePath is set */
-  if (mode & dontInit)
+  if ((configFileList != NULL && strcmp (configFileList,
+					 lastConfigFileList) == 0)
+      && !(mode & doInit))
     {
       ud->has_comp_code = 0;
       ud->has_math = 0;
@@ -1105,6 +1228,7 @@ read_configuration_file (const char *configFileList, const char
       ud->page_separator_number_first[0] = 0;
       ud->page_separator_number_first[0] = 0;
       ud->fill_pages = 0;
+      ud->mode = mode | ud->config_mode;
       return 1;
     }
   lbu_free ();
@@ -1115,7 +1239,11 @@ read_configuration_file (const char *configFileList, const char
   topMargin = 0;
   bottomMargin = 0;
   if (!(ud = malloc (sizeof (UserData))))
-    return 0;
+    {
+      lou_logPrint ("liblouisutdml: not enough memory for buffers");
+      lou_logEnd ();
+      return 0;
+    }
   memset (ud, 0, sizeof (UserData));
   ud->outbuf1_len = (sizeof (ud->outbuf1) / CHARSIZE) - 4;
   ud->outbuf2_len = (sizeof (ud->outbuf2) / CHARSIZE) - 4;
@@ -1125,6 +1253,15 @@ read_configuration_file (const char *configFileList, const char
   ud->top = -1;
   ud->style_top = -1;
   ud->emphasis = 15;
+  /* There will be configuration settings for these values*/
+  ud->cell_width = 5;
+  ud->normal_line = 8;
+  ud->wide_line = 10;
+  ud->dpi = 20.0;
+  strcpy (ud->lit_hyphen, "-");
+  strcpy (ud->comp_hyphen, "_&");
+  strcpy (ud->letsign, "\\_");
+  /*End of values*/
   for (k = document; k < notranslate; k++)
     {
       StyleType *style = new_style ((xmlChar *) semNames[k]);
@@ -1144,12 +1281,13 @@ read_configuration_file (const char *configFileList, const char
   if (configFileList == NULL)
     {
       set_paths (NULL);
-      if (!(config_compileSettings ("liblouisutdml.ini") || 
-config_compileSettings ("canonical.cfg")))
+      if (!(config_compileSettings ("liblouisutdml.ini") ||
+	    config_compileSettings ("canonical.cfg")))
 	return 0;
     }
   else
     {
+      strcpy (lastConfigFileList, configFileList);
       listLength = strlen (configFileList);
       for (k = 0; k < listLength; k++)
 	if (configFileList[k] == ',')
@@ -1198,19 +1336,45 @@ config_compileSettings ("canonical.cfg")))
   ud->braille_page_number = ud->beginning_braille_page_number;
   if (entities)
     strcat (ud->xml_header, "]>\n");
+  ud->mode = mode | ud->config_mode;
+  ud->orig_format_for = ud->format_for;
+  if (ud->format_for > utd)
+  {
+  ud->mode &= ~notUC;
+  ud->format_for = utd;
+  }
   if (ud->format_for == utd)
     {
-      const double dpi = 20.0;
-      ud->paper_width = (int)(paperWidth * dpi);
-      ud->paper_height = (int)(paperHeight * dpi);
-      ud->left_margin = (int)(leftMargin * dpi);
-      ud->right_margin = (int)(rightMargin * dpi);
-      ud->top_margin = (int)(topMargin * dpi);
-      ud->bottom_margin = (int)(bottomMargin * dpi);
+      ud->braille_pages = 1;
+      ud->paragraphs = 1;
+      ud->paper_width = (int) (paperWidth * ud->dpi);
+      ud->paper_height = (int) (paperHeight * ud->dpi);
+      ud->left_margin = (int) (leftMargin * ud->dpi);
+      ud->right_margin = (int) (rightMargin * ud->dpi);
+      ud->top_margin = (int) (topMargin * ud->dpi);
+      ud->bottom_margin = (int) (bottomMargin * ud->dpi);
     }
   else
     {
-      ud->left_margin = (int)leftMargin;
+      ud->left_margin = (int) leftMargin;
+    }
+  ud->page_left = ud->left_margin;
+  ud->page_right = ud->paper_width - ud->right_margin;
+  ud->page_top = ud->top_margin;
+  ud->page_bottom = ud->paper_height - ud->bottom_margin;
+  if (ud->format_for == utd)
+    {
+      if (ud->page_right <= 0 || ud->page_bottom <= 0)
+	{
+	  lou_logPrint
+	    ("For UTDML paper witdth and paper height must be specified.");
+	  lbu_free ();
+	  return 0;
+	}
+      ud->cells_per_line = (ud->page_right - ud->page_left) / ud->cell_width;
+      ud->lines_on_page = (ud->page_bottom - ud->page_top) / ud->normal_line;
+      ud->back_text = textDevice;
+      ud->back_line_length = 70;
     }
   return 1;
 }
