@@ -61,7 +61,8 @@ typedef enum
 {
   styleEntry = 1,
   xpathEntry = 2,
-  actionEntry = 4
+  actionEntry = 4,
+  macroEntry = 8
 } EntryType;
 
 typedef struct
@@ -72,6 +73,7 @@ typedef struct
   int semNum;
   InsertsType *inserts;
   StyleType *style;
+  unsigned char *macro;
 } HashEntry;
 
 typedef struct
@@ -158,7 +160,8 @@ hashFree (HashTable * table)
 
 static void hashInsert (HashTable * table, const unsigned char *key, int
 			type, int
-			semNum, InsertsType * inserts, StyleType * style);
+			semNum, InsertsType * inserts, StyleType * 
+			style, unsigned char *macro);
 
 void
 clean_semantic_table (void)
@@ -175,7 +178,7 @@ clean_semantic_table (void)
 	next = e->next;
 	if (!(e->type == 0 || e->type == xpathEntry))
 	  hashInsert (newSemanticTable, e->key, e->type, e->semNum, NULL,
-		      e->style);
+		      e->style, NULL);
 	free (e->key);
 	if (e->inserts != NULL)
 	  free (e->inserts);
@@ -225,7 +228,8 @@ static HashEntry *latestEntry;
 /* assumes that key is not already present! */
 static void
 hashInsert (HashTable * table, const unsigned char *key, int type, int
-	    semNum, InsertsType * inserts, StyleType * style)
+	    semNum, InsertsType * inserts, StyleType * style, unsigned 
+	    char *macro)
 {
   int i;
   if (table == NULL || key == NULL || *key == 0)
@@ -239,6 +243,7 @@ hashInsert (HashTable * table, const unsigned char *key, int type, int
   latestEntry->semNum = semNum;
   latestEntry->inserts = inserts;
   latestEntry->style = style;
+  latestEntry->macro = macro;
   table->entries[i] = latestEntry;
 }
 
@@ -294,7 +299,7 @@ find_semantic_number (const char *name)
 	  strcpy (realName, semNames[k]);
 	  strcat (realName, SEMANTICSUF);
 	  hashInsert (semanticTable, (xmlChar *) realName, actionEntry, k,
-		      NULL, NULL);
+		      NULL, NULL, NULL);
 	}
       k = 0;
       while (pseudoActions[k] != NULL)
@@ -302,7 +307,7 @@ find_semantic_number (const char *name)
 	  strcpy (realName, pseudoActions[k]);
 	  strcat (realName, SEMANTICSUF);
 	  hashInsert (semanticTable, (xmlChar *) realName,
-		      actionEntry, k + end_all + 1, NULL, NULL);
+		      actionEntry, k + end_all + 1, NULL, NULL, NULL);
 	  k++;
 	}
     }
@@ -557,7 +562,8 @@ countAttrValues (xmlChar * key)
 	return 1;
       if (curCount >= NUMCOUNTS)
 	return 0;
-      hashInsert (attrValueCountsTable, key, 0, curCount, NULL, NULL);
+      hashInsert (attrValueCountsTable, key, 0, curCount, NULL, NULL, 
+      NULL);
       curCount++;
       return 1;
     case 3:
@@ -568,7 +574,8 @@ countAttrValues (xmlChar * key)
       if (thisCount == notFound)
 	{
 	  attrValueCounts[curCount]++;
-	  hashInsert (attrValueCountsTable, key, 0, curCount, NULL, NULL);
+	  hashInsert (attrValueCountsTable, key, 0, curCount, NULL, 
+	  NULL, NULL);
 	  curCount++;
 	}
       key[lastComma] = ',';
@@ -713,7 +720,8 @@ compileLine (FileInfo * nested)
   if (actionNum == notFound && style == NULL)
     {
       semanticError (nested,
-		     "Action or style %s in column 1 not recognized", action);
+		     "Action or style '%s' in column 1 not recognized", 
+		     action);
       return 0;
     }
   if (actionNum > end_all)
@@ -797,7 +805,7 @@ compileLine (FileInfo * nested)
   else
     {
       hashInsert (semanticTable, (xmlChar *) lookFor, type, actionNum,
-		  inserts, style);
+		  inserts, style, NULL);
       nested->numEntries++;
     }
   return 1;
@@ -852,7 +860,8 @@ sem_compileFile (const char *fileName)
     }
   if (!haveAppended && !find_file (fileName, completePath))
     {
-      semanticError (NULL, "Can't find semantic-action file %s", fileName);
+      semanticError (NULL, "Can't find semantic-action file '%s'", 
+      fileName);
       haveSemanticFile = 0;
       strcpy (firstFileName, fileName);
       return 0;
@@ -870,12 +879,13 @@ sem_compileFile (const char *fileName)
       fclose (nested.in);
       if (nested.unedited)
 	semanticError (NULL,
-		       "File %s needs editing to produce good results.",
+       "File '%s' needs editing to produce good results.",
 		       nested.fileName);
     }
   else
     {
-      semanticError (NULL, "Can't open semantic-action file %s", fileName);
+      semanticError (NULL, "Can't open semantic-action file '%s'", 
+      fileName);
       return 0;
     }
   numEntries += nested.numEntries;
@@ -912,10 +922,8 @@ compile_semantic_table (xmlNode * rootElement)
   moreEntries = 0;
   numEntries = 0;
   rootName = rootElement->name;
-  if (ud->mode & dontInit)
-    return 1;
   xpathCtx = xmlXPathNewContext (rootElement->doc);
-  if (*ud->semantic_files)
+  if (ud->semantic_files != NULL)
     {
       /*Process file list */
       int listLength;
@@ -1213,6 +1221,16 @@ is_style (xmlNode * node)
     return NULL;
 }
 
+char *
+is_macro (xmlNode * node)
+{
+  HashEntry *nodeEntry = (HashEntry *) node->_private;
+  if (nodeEntry != NULL)
+    return nodeEntry->macro;
+  else
+    return NULL;
+}
+
 xmlChar *
 get_attr_value (xmlNode * node)
 {
@@ -1282,7 +1300,7 @@ addNewEntries (const xmlChar * newEntry)
     }
   if (hashLookup (newEntriesTable, newEntry) != notFound)
     return;
-  hashInsert (newEntriesTable, newEntry, 0, 0, NULL, NULL);
+  hashInsert (newEntriesTable, newEntry, 0, 0, NULL, NULL, NULL);
 }
 
 void
@@ -1339,10 +1357,11 @@ append_new_entries (void)
     }
   fclose (semOut);
   if (haveSemanticFile)
-    lou_logPrint ("%d new entries appended to %s%s.", numEntries,
+    lou_logPrint ("%d new entries appended to '%s%s'.", numEntries,
 		  filePrefix, firstFileName);
   else
-    lou_logPrint ("%d entries written to new semantic-action file %s%s.",
+    lou_logPrint (
+    "%d entries written to new semantic-action file '%s%s'.",
 		  numEntries, filePrefix, firstFileName);
   moreEntries = 0;
 }
@@ -1381,10 +1400,28 @@ new_style (xmlChar * name)
     return latestEntry->style;
   style = malloc (sizeof (StyleType));
   memset (style, 0, sizeof (StyleType));
-  hashInsert (semanticTable, key, styleEntry, 0, NULL, style);
+  style->newline_after = 1;
+  hashInsert (semanticTable, key, styleEntry, 0, NULL, style, NULL);
   return style;
 }
 
+#define MACROSUF " orcam"
+xmlChar *
+new_macro (xmlChar * name, xmlChar *body)
+{
+  char *storedBody = alloc_string (body);
+  char key[MAXNAMELEN];
+  if (!semanticTable)
+    semanticTable = hashNew ();
+  strcpy (key, name);
+  strcat (key, MACROSUF);
+  if (hashLookup (semanticTable, key) != notFound)
+    return NULL;
+  hashInsert (semanticTable, key, macroEntry, 0, NULL, NULL, 
+  storedBody);
+  return storedBody;
+}
+
 StyleType *
 lookup_style (xmlChar * name)
 {
@@ -1396,6 +1433,17 @@ lookup_style (xmlChar * name)
   return NULL;
 }
 
+unsigned char *
+lookup_macro (xmlChar * name)
+{
+  char key[MAXNAMELEN];
+  strcpy (key, name);
+  strcat (key, MACROSUF);
+  if (hashLookup (semanticTable, key) != notFound)
+    return latestEntry->macro;
+  return NULL;
+}
+
 StyleType *
 action_to_style (sem_act action)
 {
