@@ -41,6 +41,7 @@ static const struct option longopts[] = {
   {"reformat", no_argument, NULL, 'r'},
   {"poorly-formatted", no_argument, NULL, 'p'},
   {"html", no_argument, NULL, 't'},
+  {"text", no_argument, NULL, 'T'},
   {"log-file", no_argument, NULL, 'l'},
   {"config-setting", required_argument, NULL, 'C'},
   {NULL, 0, NULL, 0}
@@ -73,13 +74,13 @@ is not specified the output is sent to stdout.\n\n", stdout);
                           how to do the translation\n\
   -b, --backward      	  backward translation\n\
   -r, --reformat      	  reformat a braille file\n\
+  -T, --text		  Treat as text even if xml\n\
   -p, --poorly-formatted  translate a poorly formatted file\n\
   -t, --html              html document, not xhtml\n\
   -C, --config-setting    specify particular configuration settings\n\
                           They override any settings that are specified in a\n\
                           config file\n\
-  -l, --log-file          write errors to file2brl.log instead of stderr\n",
-   stdout);
+  -l, --log-file          write errors to file2brl.log instead of stderr\n", stdout);
 
   printf ("\n");
   printf ("\
@@ -89,8 +90,8 @@ Report bugs to <%s>.\n", PACKAGE_BUGREPORT);
 int
 main (int argc, char **argv)
 {
-  int mode = dontInit;
-  char *configFileName = NULL;
+  int mode = 0;
+  char *configFileList = NULL;
   char *inputFileName = "stdin";
   char *outputFileName = "stdout";
   char tempFileName[MAXNAMELEN];
@@ -110,7 +111,8 @@ main (int argc, char **argv)
   set_program_name (argv[0]);
 
   while ((optc =
-	  getopt_long (argc, argv, "hvf:brptlC:", longopts, NULL)) != -1)
+	  getopt_long (argc, argv, "hvf:brptlTC:", longopts, NULL)) != 
+	  -1)
     switch (optc)
       {
 	/* --help and --version exit immediately, per GNU coding standards.  */
@@ -130,12 +132,13 @@ main (int argc, char **argv)
 	mode |= htmlDoc;
 	break;
       case 'f':
-	configFileName = optarg;
+	configFileList = optarg;
 	break;
       case 'b':
       case 'p':
       case 'r':
-      case 'x':
+      case 'T':
+      case '0':
 	whichProc = optc;
 	break;
       case 'C':
@@ -177,32 +180,21 @@ main (int argc, char **argv)
     }
 
   if (whichProc == 0)
-    whichProc = 'x';
+    whichProc = '0';
   if (configSettings != NULL)
     for (k = 0; configSettings[k]; k++)
       if (configSettings[k] == '=' && configSettings[k - 1] != ' ')
 	configSettings[k] = ' ';
-  if (configFileName != NULL)
-    {
-      if ((ud = lbu_initialize (configFileName, logFileName,
-				configSettings)) == NULL)
-	exit (EXIT_FAILURE);
-    }
-  else
-    {
-      if ((ud = lbu_initialize ("preferences.cfg", logFileName,
-				configSettings)) == NULL)
-	{
-	  if ((ud = lbu_initialize ("default.cfg", logFileName,
-				    configSettings)) == NULL)
-	    exit (EXIT_FAILURE);
-	}
-    }
+  if (configFileList == NULL)
+    configFileList = "preferences.cfg";
+  if ((ud = lbu_initialize (configFileList, logFileName, 
+  configSettings)) == NULL)
+    exit (EXIT_FAILURE);
   if (strcmp (inputFileName, "stdin") != 0)
     {
       if (!(inputFile = fopen (inputFileName, "r")))
 	{
-	  lou_logPrint ("Can't open file %s.\n", inputFileName);
+	  lou_logPrint ("Can't open input file %s.\n", inputFileName);
 	  exit (EXIT_FAILURE);
 	}
     }
@@ -214,7 +206,7 @@ main (int argc, char **argv)
   if (!(tempFile = fopen (tempFileName, "w")))
     {
       lou_logPrint ("Can't open temporary file.\n");
-      exit (1);
+      exit (EXIT_FAILURE);
     }
   if (whichProc == 'p')
     {
@@ -271,10 +263,10 @@ main (int argc, char **argv)
 	  continue;
 	if (charsRead == 0)
 	  {
-	    if (ch != '<' && whichProc == 'x')
+	    if (ch != '<' && whichProc == '0')
 	      whichProc = 't';
 	    nch = fgetc (inputFile);
-	    if (!(mode & htmlDoc) && whichProc == 'x' && nch != '?')
+	    if (!(mode & htmlDoc) && whichProc == '0' && nch != '?')
 	      fprintf (tempFile, "%s\n", ud->xml_header);
 	  }
 	if (pch == '>' && ch == '<')
@@ -295,33 +287,43 @@ main (int argc, char **argv)
     switch (whichProc)
       {
       case 'b':
-	lbu_backTranslateFile (configFileName, tempFileName,
-			       outputFileName, NULL, NULL, mode);
+	lbu_backTranslateFile (configFileList, tempFileName,
+			       outputFileName, logFileName, configSettings,
+			       mode);
 	break;
       case 'r':
 	{
 	  char temp2FileName[MAXNAMELEN];
 	  strcpy (temp2FileName, lbu_getWriteablePath ());
 	  strcat (temp2FileName, "file2brl2.temp");
-	  if ((lbu_backTranslateFile (configFileName, tempFileName,
-				      temp2FileName, NULL, NULL, mode)) != 1)
-	    exit (1);
+	  if ((lbu_backTranslateFile (configFileList, tempFileName,
+				      temp2FileName, logFileName,
+				      configSettings, mode)) != 1)
+	    exit (EXIT_FAILURE);
 	  if (ud->back_text == html)
-	    lbu_translateFile (configFileName, temp2FileName,
-			       outputFileName, NULL, NULL, mode);
+	    lbu_translateFile (configFileList, temp2FileName,
+			       outputFileName, logFileName, configSettings,
+			       mode);
 	  else
-	    lbu_translateTextFile (configFileName, temp2FileName,
-				   outputFileName, NULL, NULL, mode);
+	    lbu_translateTextFile (configFileList, temp2FileName,
+				   outputFileName, logFileName,
+				   configSettings, mode);
 	}
 	break;
-      case 't':
       case 'p':
-	lbu_translateTextFile (configFileName, tempFileName,
-			       outputFileName, NULL, NULL, mode);
+	lbu_translateTextFile (configFileList, tempFileName,
+			       outputFileName, logFileName, configSettings,
+			       mode);
 	break;
-      case 'x':
-	lbu_translateFile (configFileName, tempFileName, outputFileName,
-			   NULL, NULL, mode);
+      case 'T':
+	lbu_translateTextFile (configFileList, tempFileName,
+			       outputFileName,
+			       logFileName, configSettings, mode);
+	break;
+      case 't':
+      case '0':
+	lbu_translateFile (configFileList, tempFileName, outputFileName,
+			   logFileName, configSettings, mode);
 	break;
       default:
 	lou_logPrint ("Program bug %c\n", whichProc);
